# -*-coding: utf-8 -*-
# @Time    : 2023/4/7 00:31
# @Description  : 用户认证

import pandas as pd
import os
from doupand.utils import cons as ct


def set_token(token):
    df = pd.DataFrame([token], columns=['token'])
    # user_home = os.path.expanduser('~')
    project_dir = os.path.dirname(__file__)
    fp = os.path.join(project_dir, ct.TOKEN_F_P)
    df.to_csv(fp, index=False)


def get_token():
    # user_home = os.path.expanduser('~')
    project_dir = os.path.dirname(__file__)
    fp = os.path.join(project_dir, ct.TOKEN_F_P)
    if os.path.exists(fp):
        df = pd.read_csv(fp)
        return str(df.loc[0]['token'])
    else:
        print(ct.TOKEN_ERR_MSG)
        return None



# -*-coding: utf-8 -*-
# @Time    : 2023/4/7 01:09
# @Description  : 常用变量

# token file path
TOKEN_F_P = 'dp_tk.csv'
TOKEN_ERR_MSG = '请设置DouPand的token凭证码，如果没有请访问https://doupand.com注册申请'

# http url file path
URL_F_P = 'dp_url.csv'



# -*-coding: utf-8 -*-
# @Time    : 2023/4/7 01:32
# @Description  : API数据接口

import json
import os
import requests
import time
import pandas as pd
from functools import partial
from doupand.utils import userauth
from doupand.utils import cons as ct

__all__ = ['datareader']


class DataReader:
    __token = ''
    __http_url = ''
    __distribution_url = 'http://api.doupand.com'

    def __init__(self, token='', timeout=15):
        """
        :param token: API的TOKEN，用于用户认证
        :param timeout:
        """
        self.__token = token
        self.__timeout = timeout
        self.__url_expire_time = int(time.time())

    @property
    def __query_token(self):
        """请求口令token"""
        if self.__token == '' or self.__token is None:
            token = userauth.get_token()
            if token is not None and token != '':
                self.__token = token
            else:
                raise Exception('DataReader Init Error!')

        return self.__token

    @property
    def __query_url(self):
        """请求URL"""
        cur_time = int(time.time())
        if self.__http_url and cur_time < self.__url_expire_time:
            return self.__http_url
        else:
            project_dir = os.path.dirname(__file__)
            fp = os.path.join(project_dir, ct.URL_F_P)

            if os.path.exists(fp):
                df = pd.read_csv(fp)
                if cur_time < int(df.loc[0]['expire_time']):
                    self.__url_expire_time = int(df.loc[0]['expire_time'])
                    return str(df.loc[0]['http_url'])

            res = requests.get(self.__distribution_url, timeout=10, headers={'Connection': 'close'})
            result = res.json()
            if result['status'] == "success":
                self.__http_url = res.json()['result']
                self.__url_expire_time = cur_time + 12 * 60 * 60
                df = pd.DataFrame([[self.__http_url, self.__url_expire_time]], columns=['http_url', 'expire_time'])
                df.to_csv(fp, index=False)
                return self.__http_url
            else:
                raise Exception("初始化请求失败，请重试！")

    def query(self, api_name, fields='', **kwargs):
        """
        请求API
        :param api_name: API名称
        :param fields: 返回字段
        :param kwargs: 参数
        :return:
        """
        req_params = {
            'api_name': api_name,
            'token': self.__query_token,
            'params': kwargs,
            'fields': fields
        }

        res = requests.post(self.__query_url, json=req_params, timeout=self.__timeout, headers={'Connection': 'close'})
        result = json.loads(res.text)
        if result['code'] != 0:
            raise Exception(result['msg'])
        data = result['data']
        columns = data['columns']
        values = data['values']

        return pd.DataFrame(values, columns=columns)

    def __getattr__(self, name):
        """
        直接将属性名称作为api_name传入query方法
        :param name:
        :return:
        """
        return partial(self.query, name)


datareader = DataReader()



# -*-coding: utf-8 -*-
# @Time    : 2023/4/8 02:39
# @Description  : PIPY INFO

from setuptools import find_packages, setup
import doupand

long_desc = """
DouPand
===============

* easy to use as most of the data returned are pandas DataFrame objects
* can be easily saved as csv, excel or json files
* can be inserted into MySQL or Mongodb

Target Users
--------------

* financial market analyst of China
* learners of financial data analysis with pandas/NumPy
* people who are interested in China financial data

Installation
--------------

    pip install doupand

Upgrade
---------------

    pip install doupand --upgrade

Quick Start
--------------

::

    import doupand as dp

    print(dp.datareader.ashare_description())

return::

          s_dp_code  s_code  ... s_exchmarket s_listboard
    0     000001.SZ  000001  ...         SZSE          主板
    1     000002.SZ  000002  ...         SZSE          主板
    2     000003.SZ  000003  ...         SZSE          主板
    3     000004.SZ  000004  ...         SZSE          主板
    4     000005.SZ  000005  ...         SZSE          主板
    ...         ...     ...  ...          ...         ...
    5329  873169.BJ  873169  ...          BSE         北交所
    5330  873223.BJ  873223  ...          BSE         北交所
    5331  873305.BJ  873305  ...          BSE         北交所
    5332  873339.BJ  873339  ...          BSE         北交所
    5333  873527.BJ  873527  ...          BSE         北交所
"""

setup(
    name='doupand',
    version=doupand.__version__,
    description='A simple and easy-to-use financial data interface library built for normal investors!',
    long_description=long_desc,
    author='DouBro',
    author_email='doupand@163.com',
    packages=find_packages(),
    platforms=["all"],
    license='BSD',
    keywords='Financial Data Interface',
    url='https://doupand.com',
    install_requires=["pandas", "requests"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'License :: OSI Approved :: BSD License'
    ],
    include_package_data=True,
    package_data={'': ['*.csv', '*.txt']},
)
